@@ -21,8 +21,6 @@
  */
 #include "jobtracker.h"
 
-#include <KIconLoader>
-#include <QTimer>
 #include <KDebug>
 
 JobTrackerWidget::JobTrackerWidget(QWidget *parent)
@@ -32,17 +30,12 @@ JobTrackerWidget::JobTrackerWidget(QWidget *parent)
 }
 
 JobTracker::JobTracker(QWidget *parent)
-        : KAbstractWidgetJobTracker(parent), m_currentJob(0)
+        : KAbstractWidgetJobTracker(parent)
 {
     m_ui = new JobTrackerWidget(parent);
     resetUi();
 }
 
-KJob* JobTracker::currentJob() const
-{
-    return m_currentJob;
-}
-
 void JobTracker::description(KJob *job, const QString &title, const QPair< QString, QString > &f1, const QPair< QString, QString > &f2)
 {
     Q_UNUSED(job)
@@ -73,7 +66,6 @@ void JobTracker::registerJob(KJob *job)
     m_ui->show();
     m_ui->informationLabel->hide();
     m_ui->progressBar->show();
-    m_currentJob = job;
 }
 
 void JobTracker::percent(KJob *job, unsigned long  percent)
@@ -86,10 +78,8 @@ void JobTracker::percent(KJob *job, unsigned long  percent)
 
 void JobTracker::unregisterJob(KJob *job)
 {
-    kDebug();
-    resetUi();
-    m_currentJob = 0;
     KJobTrackerInterface::unregisterJob(job);
+    resetUi();
 }
 
 void JobTracker::resetUi()
@@ -101,15 +91,6 @@ void JobTracker::resetUi()
     m_ui->progressBar->setMinimum(0);
 }
 
-void JobTracker::finished(KJob *job)
-{
-    QTimer::singleShot(1500, this, SLOT(resetUi()));
-    m_ui->informationLabel->setText(i18n("Operation finished."));
-    m_ui->informationLabel->show();
-    m_ui->progressBar->hide();
-    KJobTrackerInterface::unregisterJob(job);
-}
-
 QWidget* JobTracker::widget(KJob *)
 {
     return m_ui;
