@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2002 Jean-Baptiste Mardelle <bj@altern.org>
  * Copyright (C) 2007,2008 Rolf Eike Beer <kde@opensource.sf-tec.de>
  */
@@ -41,7 +41,6 @@ public:
 	int m_textlength;
 	QString m_message;
 	QString m_signID;
-	QString m_log;
 	QStringList m_userIDs;
 	QStringList m_gpgopts;
 	QByteArray m_tmpmessage;
@@ -55,18 +54,46 @@ public:
 	bool symPassphrase();
 	bool gpgPassphrase();
 	void signFile(const KUrl &);
+
+	void appendLog(const QByteArray &a);
+	QString log() const;
+
+private:
+	bool m_consoleUtf8;
+	QString m_log;
 };
 
-KGpgTextInterfacePrivate::KGpgTextInterfacePrivate()
+static bool isUtf8Lang(const QByteArray &lc)
 {
-	m_anonymous = false;
-	m_badmdc = false;
-	m_ok = false;
-	m_textlength = -1;
-	m_step = 3;
-	m_badpassword = false;
-	m_signmiss = false;
-	m_forcefail = false;
+	return lc.endsWith("UTF-8");
+}
+
+KGpgTextInterfacePrivate::KGpgTextInterfacePrivate()
+	: m_ok(false),
+	m_badmdc(false),
+	m_badpassword(false),
+	m_anonymous(false),
+	m_signmiss(false),
+	m_forcefail(false),
+	m_step(3),
+	m_textlength(-1),
+	m_consoleUtf8(true)
+{
+	const QByteArray lcMess = qgetenv("LC_MESSAGES");
+
+	if (!lcMess.isEmpty()) {
+		m_consoleUtf8 = isUtf8Lang(lcMess);
+	} else {
+		const QByteArray lcAll = qgetenv("LC_ALL");
+		if (!lcAll.isEmpty()) {
+			m_consoleUtf8 = isUtf8Lang(lcAll);
+		} else {
+			const QByteArray lang = qgetenv("LANG");
+			if (!lang.isEmpty())
+				m_consoleUtf8 = isUtf8Lang(lang);
+		}
+	}
+	
 }
 
 void
@@ -111,7 +138,8 @@ KGpgTextInterfacePrivate::gpgPassphrase()
 	if (m_anonymous)
 		passdlgmessage = i18n("<p><b>No user id found</b>. Trying all secret keys.</p>");
 	if ((m_step < 3) && !m_anonymous)
-		passdlgmessage = i18n("<p><b>Bad passphrase</b>. You have %1 tries left.</p>", m_step);
+		passdlgmessage = i18np("<p><b>Bad passphrase</b>. You have 1 try left.</p>",
+		                       "<p><b>Bad passphrase</b>. You have %1 tries left.</p>", m_step);
 	if (m_userIDs.isEmpty())
 		passdlgmessage += i18n("Enter passphrase");
 	else
@@ -132,16 +160,30 @@ KGpgTextInterfacePrivate::signFile(const KUrl &file)
 {
 	*m_process << "--command-fd=0" << "-u" << m_signID.simplified().toLocal8Bit();
 
-	for (QStringList::ConstIterator it = m_gpgopts.constBegin(); it != m_gpgopts.constEnd(); ++it)
-		*m_process << QFile::encodeName(*it);
+	*m_process << m_gpgopts;
 
 	if (m_gpgopts.contains("--detach-sign") && !m_gpgopts.contains("--output"))
-		*m_process << "--output" << QFile::encodeName(file.path() + ".sig");
-	*m_process << QFile::encodeName(file.path());
+		*m_process << "--output" << file.path() + ".sig";
+	*m_process << file.path();
 
 	m_process->start();
 }
 
+void
+KGpgTextInterfacePrivate::appendLog(const QByteArray &a)
+{
+	if (m_consoleUtf8)
+		m_log += QString::fromUtf8(a);
+	else
+		m_log += QString::fromLocal8Bit(a);
+}
+
+QString
+KGpgTextInterfacePrivate::log() const
+{
+	return m_log;
+}
+
 KGpgTextInterface::KGpgTextInterface(QObject *parent)
 	: QObject(parent), d(new KGpgTextInterfacePrivate)
 {
@@ -165,19 +207,18 @@ KGpgTextInterface::encryptText(const QString &text, const QStringList &userids,
 	else
 		d->m_message = text.toUtf8();
 
-	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
-		*d->m_process << QFile::encodeName(*it);
+	*d->m_process << options;
 
 	if (userids.isEmpty()) {
 		*d->m_process << "-c";
 	} else {
 		*d->m_process << "-e";
-		for (QStringList::ConstIterator it = userids.constBegin(); it != userids.constEnd(); ++it)
-			*d->m_process << "--recipient" << *it;
+		foreach (const QString &uid, userids)
+			*d->m_process << "--recipient" << uid;
 	}
 
-	connect(d->m_process, SIGNAL(readReady(GPGProc *)), this, SLOT(encryptTextProcess()));
-	connect(d->m_process, SIGNAL(processExited(GPGProc *)), this, SLOT(encryptTextFin()));
+	connect(d->m_process, SIGNAL(readReady()), this, SLOT(encryptTextProcess()));
+	connect(d->m_process, SIGNAL(processExited()), this, SLOT(encryptTextFin()));
 	d->m_process->start();
 }
 
@@ -188,9 +229,9 @@ KGpgTextInterface::encryptTextProcess()
 	QString line;
 
 	while ( (items = d->m_process->readln(line)) >= 0 ) {
-		if (line.startsWith("[GNUPG:] ")) {
+		if (line.startsWith(QLatin1String("[GNUPG:] "))) {
 			line.remove(0, 9);
-			if (line.startsWith("BEGIN_ENCRYPTION")) {
+			if (line.startsWith(QLatin1String("BEGIN_ENCRYPTION"))) {
 				d->m_process->write(d->m_message.toAscii());
 				d->m_process->closeWriteChannel();
 				d->m_message.clear();
@@ -209,21 +250,20 @@ KGpgTextInterface::encryptTextProcess()
 void
 KGpgTextInterface::encryptTextFin()
 {
-	emit txtEncryptionFinished(QString::fromUtf8(d->m_message.toAscii()).trimmed(), this);
+	emit txtEncryptionFinished(d->m_message.trimmed());
 }
 
 void
 KGpgTextInterface::decryptText(const QString &text, const QStringList &options)
 {
-	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
-		*d->m_process << QFile::encodeName(*it);
+	*d->m_process << options;
 
 	*d->m_process << "-d";
 
 	d->m_process->setOutputChannelMode(KProcess::SeparateChannels);
 	connect(d->m_process, SIGNAL(readyReadStandardError()), this, SLOT(decryptTextStdErr()));
 	connect(d->m_process, SIGNAL(lineReadyStandardOutput()), this, SLOT(decryptTextStdOut()));
-	connect(d->m_process, SIGNAL(processExited(GPGProc *)), this, SLOT(decryptTextFin()));
+	connect(d->m_process, SIGNAL(processExited()), this, SLOT(decryptTextFin()));
 	d->m_process->start();
 
 	d->m_process->write(text.toAscii());
@@ -235,9 +275,10 @@ KGpgTextInterface::decryptTextStdErr()
 	QByteArray a;
 
 	while (d->m_process->readLineStandardError(&a))
-		d->m_log.append(a + '\n');
+		d->appendLog(a + '\n');
 }
 
+//krazy:cond=strings
 void
 KGpgTextInterface::decryptTextStdOut()
 {
@@ -261,7 +302,7 @@ KGpgTextInterface::decryptTextStdOut()
 			} else if (line.startsWith("BEGIN_DECRYPTION")) {
 				d->m_process->closeWriteChannel();
 			} else if (line.startsWith("PLAINTEXT_LENGTH")) {
-				d->m_textlength = line.mid(line.indexOf("PLAINTEXT_LENGTH") + 16).toInt();
+				d->m_textlength = line.mid(16).toInt();
 			} else if (line.startsWith("DECRYPTION_OKAY")) {
 				d->m_ok = true;
 				// this happens if the encrypted text does not end with '\n'
@@ -298,17 +339,18 @@ KGpgTextInterface::decryptTextStdOut()
 		}
 	}
 }
+//krazy:endcond=strings
 
 void
 KGpgTextInterface::decryptTextFin()
 {
 	if ((d->m_ok) && (!d->m_badmdc)) {
-		emit txtDecryptionFinished(d->m_tmpmessage, this);
+		emit txtDecryptionFinished(d->m_tmpmessage);
 	} else if (d->m_badmdc) {
 		KMessageBox::sorry(0, i18n("Bad MDC detected. The encrypted text has been manipulated."));
-		emit txtDecryptionFailed(d->m_log, this);
+		emit txtDecryptionFailed(d->log());
 	} else
-		emit txtDecryptionFailed(d->m_log, this);
+		emit txtDecryptionFailed(d->log());
 }
 
 void
@@ -320,16 +362,16 @@ KGpgTextInterface::signText(const QString &text, const QString &userid, const QS
 	else
 		d->m_message = text.toUtf8();
 
-	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
-		*d->m_process << QFile::encodeName(*it);
+	*d->m_process << options;
 
 	*d->m_process << "--clearsign" << "-u" << userid;
 
 	connect(d->m_process, SIGNAL(lineReadyStandardOutput()), this, SLOT(signTextProcess()));
-	connect(d->m_process, SIGNAL(processExited(GPGProc *)), this, SLOT(signTextFin()));
+	connect(d->m_process, SIGNAL(processExited()), this, SLOT(signTextFin()));
 	d->m_process->start();
 }
 
+//krazy:cond=strings
 void
 KGpgTextInterface::signTextProcess()
 {
@@ -355,17 +397,18 @@ KGpgTextInterface::signTextProcess()
 			d->m_message += line + '\n';
 	}
 }
+//krazy:endcond=strings
 
 void
 KGpgTextInterface::signTextFin()
 {
 	if (d->m_badpassword) {
-		emit txtSigningFailed(d->m_message, this);
+		emit txtSigningFailed(d->m_message);
 		d->m_message.clear();
 	} else if (!d->m_message.isEmpty()) {
-		emit txtSigningFinished(d->m_message.trimmed(), this);
+		emit txtSigningFinished(d->m_message.trimmed());
 	} else {
-		emit txtSigningFinished(QString(), this);
+		emit txtSigningFinished(QString());
 	}
 }
 
@@ -380,8 +423,8 @@ KGpgTextInterface::verifyText(const QString &text)
 
 	*d->m_process << "--verify";
 
-	connect(d->m_process, SIGNAL(readReady(GPGProc *)), this, SLOT(readVerify()));
-	connect(d->m_process, SIGNAL(processExited(GPGProc *)), this, SLOT(verifyTextFin()));
+	connect(d->m_process, SIGNAL(readReady()), this, SLOT(readVerify()));
+	connect(d->m_process, SIGNAL(processExited()), this, SLOT(verifyTextFin()));
 	d->m_process->start();
 
 	d->m_process->write(d->m_message.toAscii());
@@ -393,12 +436,12 @@ void
 KGpgTextInterface::verifyTextFin()
 {
 	if (d->m_signmiss) {
-		emit txtVerifyMissingSignature(d->m_signID, this);
+		emit txtVerifyMissingSignature(d->m_signID);
 	} else {
 		if (d->m_signID.isEmpty())
 		d->m_signID = i18n("No signature found.");
 
-		emit txtVerifyFinished(d->m_signID, d->m_message, this);
+		emit txtVerifyFinished(d->m_signID, d->m_message);
 	}
 }
 
@@ -407,27 +450,25 @@ KGpgTextInterface::encryptFile(const QStringList &encryptkeys, const KUrl &srcur
 {
 	d->m_file = srcurl;
 
-	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
-		*d->m_process << QFile::encodeName(*it);
+	*d->m_process << options;
 
 	if (!options.contains("--output"))
-		*d->m_process << "--output" << QFile::encodeName(desturl.path());
+		*d->m_process << "--output" << desturl.path();
 
 	if (!symetrical) {
 		*d->m_process << "-e";
-		for (QStringList::ConstIterator it = encryptkeys.constBegin(); it != encryptkeys.constEnd(); ++it)
-		*d->m_process << "--recipient" << *it;
+		foreach (const QString &enckey, encryptkeys)
+			*d->m_process << "--recipient" << enckey;
 	} else
 		*d->m_process << "-c";
 
-	*d->m_process << QFile::encodeName(srcurl.path());
+	*d->m_process << srcurl.path();
 
-	connect(d->m_process, SIGNAL(readReady(GPGProc *)), this, SLOT(fileReadEncProcess()));
-	connect(d->m_process, SIGNAL(processExited(GPGProc *)), this, SLOT(fileEncryptFin()));
+	connect(d->m_process, SIGNAL(readReady()), this, SLOT(fileReadEncProcess()));
+	connect(d->m_process, SIGNAL(processExited()), this, SLOT(fileEncryptFin()));
 	d->m_process->start();
 }
 
-// TODO : there is a bug when we want to encrypt a file
 void
 KGpgTextInterface::fileReadEncProcess()
 {
@@ -435,7 +476,7 @@ KGpgTextInterface::fileReadEncProcess()
 
 	while (d->m_process->readln(line) >= 0) {
 		kDebug(2100) << line ;
-		if (line.startsWith("[GNUPG:]")) {
+		if (line.startsWith(QLatin1String("[GNUPG:]"))) {
 			if (line.contains("BEGIN_ENCRYPTION")) {
 				emit processstarted(d->m_file.path());
 			} else if (line.contains("GET_" )) {
@@ -462,27 +503,9 @@ void
 KGpgTextInterface::fileEncryptFin()
 {
 	if (d->m_ok)
-		emit fileEncryptionFinished(d->m_file, this);
+		emit fileEncryptionFinished(d->m_file);
 	else
-		emit errorMessage(d->m_message, this);
-}
-
-// decrypt file to text
-void
-KGpgTextInterface::KgpgDecryptFileToText(const KUrl &srcUrl, const QStringList &options)
-{
-	*d->m_process << "--no-batch" << "-o" << "-";
-
-	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
-		*d->m_process << QFile::encodeName(*it);
-
-	*d->m_process << "-d" << QFile::encodeName(srcUrl.path());
-
-	d->m_process->setOutputChannelMode(KProcess::SeparateChannels);
-	connect(d->m_process, SIGNAL(readyReadStandardError()), this, SLOT(decryptTextStdErr()));
-	connect(d->m_process, SIGNAL(readReady(GPGProc *)), this, SLOT(decryptTextStdOut()));
-	connect(d->m_process, SIGNAL(processExited(GPGProc *)), this, SLOT(decryptTextFin()));
-	d->m_process->start();
+		emit errorMessage(d->m_message);
 }
 
 void
@@ -504,6 +527,7 @@ KGpgTextInterface::decryptFile(const KUrl &src, const KUrl &dest, const QStringL
 	d->m_process->start();
 }
 
+//krazy:cond=strings
 void
 KGpgTextInterface::decryptFileProcess()
 {
@@ -524,7 +548,7 @@ KGpgTextInterface::decryptFileProcess()
 					d->m_process->write("Yes\n");
 				} else if ((line.contains("passphrase.enter"))) {
 					if (d->gpgPassphrase()) {
-						emit decryptFileFinished(1, this);
+						emit decryptFileFinished(1);
 						return;
 					}
 				} else {
@@ -536,6 +560,7 @@ KGpgTextInterface::decryptFileProcess()
 		}
 	}
 }
+//krazy:endcond=strings
 
 void
 KGpgTextInterface::decryptFileFin(int res, QProcess::ExitStatus status)
@@ -543,21 +568,21 @@ KGpgTextInterface::decryptFileFin(int res, QProcess::ExitStatus status)
 	if (status == QProcess::CrashExit) {
 		if (d->m_dest.isLocalFile())
 			QFile(d->m_dest.toLocalFile()).remove();
-		emit decryptFileFinished(2, this);
+		emit decryptFileFinished(2);
 		return;
 	}
 
 	if (res != 0) {
 		if (d->m_dest.isLocalFile())
 			QFile(d->m_dest.toLocalFile()).remove();
-		emit decryptFileFinished(3, this);
+		emit decryptFileFinished(3);
 		return;
 	}
 
 	if (d->m_message.contains("DECRYPTION_OKAY") && d->m_message.contains("END_DECRYPTION"))
-		emit decryptFileFinished(0, this);
+		emit decryptFileFinished(0);
 	else
-		emit decryptFileFinished(4, this);
+		emit decryptFileFinished(4);
 }
 
 void
@@ -567,14 +592,15 @@ KGpgTextInterface::KgpgVerifyFile(const KUrl &sigUrl, const KUrl &srcUrl)
 
 	*d->m_process << "--verify";
 	if (!srcUrl.isEmpty())
-		*d->m_process << QFile::encodeName(srcUrl.path());
-	*d->m_process << QFile::encodeName(sigUrl.path());
+		*d->m_process << srcUrl.path();
+	*d->m_process << sigUrl.path();
 
 	connect(d->m_process, SIGNAL(lineReadyStandardOutput()), this, SLOT(readVerify()));
-	connect(d->m_process, SIGNAL(processExited(GPGProc *)), this, SLOT(verifyfin()));
+	connect(d->m_process, SIGNAL(processExited()), this, SLOT(verifyfin()));
 	d->m_process->start();
 }
 
+//krazy:cond=strings
 void
 KGpgTextInterface::readVerify()
 {
@@ -610,6 +636,7 @@ KGpgTextInterface::readVerify()
 		}
 	}
 }
+//krazy:endcond=strings
 
 void
 KGpgTextInterface::verifyfin()
@@ -618,7 +645,8 @@ KGpgTextInterface::verifyfin()
 		if (d->m_signID.isEmpty())
 			d->m_signID = i18n("No signature found.");
 
-		(void) new KgpgDetailedInfo(0, d->m_signID, d->m_message);
+		(void) new KgpgDetailedInfo(qobject_cast<QWidget *>(parent()), d->m_signID, d->m_message,
+				QStringList(), i18nc("Caption of message box", "Verification Finished"));
 	} else {
 		if (KMessageBox::questionYesNo(0,
 					i18n("<qt><b>Missing signature:</b><br />Key id: %1<br /><br />Do you want to import this key from a keyserver?</qt>", d->m_signID),
@@ -646,8 +674,8 @@ KGpgTextInterface::signFilesBlocking(const QString &keyID, const KUrl::List &src
 	d->m_signID = keyID;
 	d->m_gpgopts = options;
 
-	for (int i = 0; i < srcUrls.count(); i++) {
-		d->signFile(srcUrls.at(i));
+	foreach (const KUrl &url, srcUrls) {
+		d->signFile(url);
 		d->m_process->waitForFinished(-1);
 		d->m_process->resetProcess();
 	}
@@ -676,5 +704,5 @@ KGpgTextInterface::slotSignFinished(int err)
 	if (err != 0)
 		d->m_errfiles << d->m_files.at(d->m_step - 1);
 
-	emit fileSignFinished(this, d->m_errfiles);
+	emit fileSignFinished(d->m_errfiles);
 }
