@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Rolf Eike Beer <kde@opensource.sf-tec.de>
+ * Copyright (C) 2008,2009 Rolf Eike Beer <kde@opensource.sf-tec.de>
  */
 
 /***************************************************************************
@@ -15,30 +15,28 @@
 #define KGPGCHANGEEXPIRE_H
 
 #include <QObject>
-#include <QDate>
+#include <QDateTime>
 
-#include "kgpgtransaction.h"
+#include "kgpgeditkeytransaction.h"
 
 /**
- * \brief change the key lifetime
+ * @brief change the key lifetime
  */
-class KGpgChangeExpire: public KGpgTransaction {
+class KGpgChangeExpire: public KGpgEditKeyTransaction {
 	Q_OBJECT
 
+	Q_DISABLE_COPY(KGpgChangeExpire)
 public:
-	KGpgChangeExpire(QObject *parent, const QString &keyid, const QDate &date);
+	KGpgChangeExpire(QObject *parent, const QString &keyid, const QDateTime &date);
 	virtual ~KGpgChangeExpire();
 
-	void setDate(const QDate &date);
+	void setDate(const QDateTime &date);
 
 protected:
-	virtual bool preStart();
 	virtual bool nextLine(const QString &line);
-	virtual void finish();
 
 private:
-	QDate m_date;
-	int m_step;
+	QDateTime m_date;
 };
 
 #endif // KGPGCHANGEEXPIRE_H
