@@ -27,6 +27,8 @@
 #include "columnsview.h"
 // Okteta core
 #include "addressrange.h"
+// Qt
+#include <QtGui/QClipboard>
 
 class QMimeData;
 class QByteArray;
@@ -52,12 +54,16 @@ class AbstractByteArrayViewPrivate;
 // so it's AbstractByteArrayView < ColumnsView < {ByteArrayRowView,ByteArrayColumnView}
 class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
 {
+  friend class ByteArrayJanusView;
   friend class KTabController;
   friend class KNavigator;
   friend class KEditor;
   friend class KValueEditor;
   friend class KCharEditor;
   friend class Dropper;
+  friend class MouseNavigator;
+  friend class MousePaster;
+  friend class Dragger;
 
   Q_OBJECT
   Q_PROPERTY( bool OverwriteMode READ isOverwriteMode WRITE setOverwriteMode )
@@ -76,6 +82,17 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
   Q_PROPERTY( CodingTypes VisibleCodings READ visibleCodings WRITE setVisibleCodings )
   Q_ENUMS( ValueCoding CharCoding LayoutStyle CodingTypes )
 
+  Q_PROPERTY( bool ByteTypeColored READ isByteTypeColored WRITE setByteTypeColored )
+  // value column
+  Q_PROPERTY( ValueCoding Coding READ valueCoding WRITE setValueCoding )
+  Q_PROPERTY( int ByteSpacingWidth READ byteSpacingWidth WRITE setByteSpacingWidth )
+  Q_PROPERTY( int NoOfGroupedBytes READ noOfGroupedBytes WRITE setNoOfGroupedBytes )
+  Q_PROPERTY( int GroupSpacingWidth READ groupSpacingWidth WRITE setGroupSpacingWidth )
+  Q_PROPERTY( int BinaryGapWidth READ binaryGapWidth WRITE setBinaryGapWidth )
+  // char column
+  Q_PROPERTY( bool ShowNonprinting READ showsNonprinting WRITE setShowsNonprinting )
+  Q_PROPERTY( QChar SubstituteChar READ substituteChar WRITE setSubstituteChar )
+
   public:
     enum ValueCoding { HexadecimalCoding=0, DecimalCoding=1, OctalCoding=2, BinaryCoding=3, MaxCodingId=0xFFFF };
     enum CharCoding { LocalEncoding=0, ISO8859_1Encoding=1, EBCDIC1047Encoding=2,
@@ -112,6 +129,8 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     CodingTypes visibleCodings() const;
     CodingTypeId activeCoding() const;
 
+    bool offsetColumnVisible() const;
+
     /** returns true if there is a selected range in the array */
     bool hasSelectedData() const;
     AddressRange selection() const;
@@ -131,8 +150,6 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     double zoomLevel() const;
 
   public: // value access API
-    virtual bool offsetColumnVisible() const = 0;
-
   // value column
     virtual int/*PixelX*/ byteSpacingWidth() const = 0;
     virtual int noOfGroupedBytes() const = 0;
@@ -158,9 +175,6 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
   public:
     virtual void setByteArrayModel( Okteta::AbstractByteArrayModel* byteArrayModel );
 
-    /** switches the Offset column on/off */
-    virtual void toggleOffsetColumn( bool offsetColumnVisible ) = 0;
-
   // setting parameters
   // value column parameters
     /** sets the spacing between the bytes in the value column
@@ -250,6 +264,9 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     /** */
     void setVisibleCodings( int visibleCodings );
 
+    /** switches the Offset column on/off */
+    void toggleOffsetColumn( bool offsetColumnVisible );
+
   public:
     /** 
       * @return deep copy of the selected data 
@@ -360,6 +377,8 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     void finishByteEdit();
     void emitSelectionSignals();
     void updateChanged();
+    void copyToClipboard( QClipboard::Mode mode ) const;
+    void pasteFromClipboard( QClipboard::Mode mode );
 
   protected:
     const Okteta::ValueCodec* valueCodec() const;
@@ -370,6 +389,10 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
 
   protected: // QWidget API
     virtual void keyPressEvent( QKeyEvent* keyEvent );
+    virtual void mousePressEvent( QMouseEvent* mousePressEvent );
+    virtual void mouseReleaseEvent( QMouseEvent* mouseReleaseEvent );
+    virtual void mouseMoveEvent( QMouseEvent* mouseMoveEvent );
+    virtual void mouseDoubleClickEvent( QMouseEvent* mouseDoubleClickEvent );
     /// reimplemented to catch Tab and BackTab keys, which otherwise gets stolen
     virtual bool event( QEvent* event );
     virtual void showEvent( QShowEvent* showEvent );
@@ -380,6 +403,7 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     virtual void dragMoveEvent( QDragMoveEvent* dragMoveEvent) ;
     virtual void dragLeaveEvent( QDragLeaveEvent* dragLeaveEvent );
     virtual void dropEvent( QDropEvent* dropEvent );
+//    virtual void contextMenuEvent( QContextMenuEvent* contextMenuEvent );
 
   protected: // QAbstractScrollArea API
     virtual void wheelEvent( QWheelEvent *e );
@@ -397,10 +421,12 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
   private:
     Q_DECLARE_PRIVATE( AbstractByteArrayView )
     Q_PRIVATE_SLOT( d_func(), void adaptController() )
+    Q_PRIVATE_SLOT( d_func(), void blinkCursor() )
     Q_PRIVATE_SLOT( d_func(), void onByteArrayReadOnlyChange(bool isByteArrayReadOnly) )
     Q_PRIVATE_SLOT( d_func(), void onContentsChanged( const Okteta::ArrayChangeMetricsList& changeList ) )
     Q_PRIVATE_SLOT( d_func(), void onBookmarksChange( const QList<Okteta::Bookmark>& bookmarks ) )
     Q_PRIVATE_SLOT( d_func(), void onRevertedToVersionIndex( int versionIndex ) )
+//     Q_PRIVATE_SLOT( d_func(), void onClipboardChanged() )
 };
 
 }
